<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Table.php 1177 2004-12-21 12:40:10Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage Table
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal.php';

/**
 * An abstract class defining the interface for creating Max database tables.
 *
 * Note that only permanent tables are created with the table prefix defined in the
 * configuration .ini file - temporary tables do NOT use the table prefix. This is 
 * because temporary tables are not defined in the table array in the configuration
 * .ini file, and as such, must be referenced in the Data Access Layer code
 * directly, and this is easier to do if the prefix doesn't have to be prepended
 * in order to do so.
 *
 * @abstract
 *
 */
class Dal_Table
{
    var $conf;
    var $dal;
    var $tables;
    var $splitTables;
    
    /**
     * The constructor method.
     *
     * @param string $file The name of the SQL file to parse for
     *                     the table definitions.
     */
    function Dal_Table($file)
    {
        $this->conf =& $GLOBALS['_MAX']['CONF'];
        $this->dal = Dal::singleton();
        // Parse the SQL file
        if (!($fp = fopen($file, 'r'))) {
            debug('Unable to open the Max table definition file: ' . $file, PEAR_LOG_ERROR);
            debug('Aborting script execution', PEAR_LOG_ERROR);
            exit();
        }
        $sql = '';
        while (!feof($fp)) {
            $line = fgets($fp);
            $line = trim($line);
            if (preg_match('/^-- SPLIT TABLE (\w+);?/', $line, $matches)) {
                // Line is a comment defining that the table may be split
                $tableName = $matches[1];
                $this->splitTables[$tableName] = true;
                continue;
            }
            $comment = substr($line, 0, 2);
            if ($comment === '--') {
                // Line is a general comment, ignore
                continue;
            }
            $sql = $sql . $line;
            if (preg_match('/;$/', $sql)) {
                // The table create SQL has been completed, store it, with the 
                // appropriate table prefix and TYPE definition
                if (preg_match('/^(CREATE TABLE IF NOT EXISTS )(\w+)(.*);/i', $sql, $matches)) {
                    $tableName = $matches[2];
                    $this->tables[$tableName] = $matches[1] . $this->conf['table']['prefix'] . $matches[2] . $matches[3] . ' TYPE=' . $this->conf['table']['type'];
                    $sql = '';
                } else if (preg_match('/^(CREATE TEMPORARY TABLE IF NOT EXISTS )(\w+)(.*);/i', $sql, $matches)) {
                    $tableName = $matches[2];
                    $this->tables[$tableName] = $matches[1] . $matches[2] . $matches[3] . ' TYPE=' . $this->conf['table']['type'];
                    $sql = '';
                } else {
                    // Bad SQL, abort
                    debug('Bad SQL code:' . $sql, PEAR_LOG_ERROR);
                    debug('Aborting script execution', PEAR_LOG_ERROR);
                    exit();
                }
            }
        }
        fclose($fp);
    }
    
    /**
     * A factory method for creating tables.
     *
     * @param string $table The name of the table to create.
     * @param Date $date An optional date for creating split tables.
     *                   Will use current date if required and not
     *                   supplied.
     * @return string The name of the table created.
     */
    function createTable($table, $date = NULL)
    {
        if (isset($this->tables[$table])) {
            // Are split tables in operation, and is the table designed to be split?
            if (($this->conf['table']['split']) && ($this->splitTables[$table])) {
                // Alter the SQL to create the appropriate split table
                if ($date == NULL) {
                    $date = new Date();
                }
                if (preg_match('/^(CREATE TABLE IF NOT EXISTS )(' . $this->conf['table_prefix'] . ')(\w+)(.*)/i', $this->tables[$table], $matches)) {
                    $tableName = $matches[2] . $matches[3] . '_' . $date->format('%Y%m%d');
                    $query = $matches[1] . $tableName  . $matches[4];
                } else if (preg_match('/^(CREATE TEMPORARY TABLE IF NOT EXISTS )(\w+)(.*)/i', $this->tables[$table], $matches)) {
                    $tableName = $matches[2] . '_' . $date->format('%Y%m%d');
                    $query = $matches[1] . $tableName  . $matches[3];
                }
            } else {
                // Use the default SQL
                $tableName = $table;
                $query = $this->tables[$table];
            }
            debug('Creating the ' . $tableName . ' table', PEAR_LOG_DEBUG);
            $this->dal->execute($query, $result);
            return $tableName;
        } else {
            // The table name requested does not exist
            debug('The table ' . $table . ' is not defined.', PEAR_LOG_ERROR);
            debug('Aborting script execution', PEAR_LOG_ERROR);
            exit();
        }
    }
    
    /**
     * A method for dropping tables.
     *
     * @param string $table The table name to drop. Use the table name WITH prefix
     *                      (i.e. obtained from parsing the configuration .ini file).
     */
    function dropTable($table)
    {
        $query = 'DROP TABLE ' . $table;
        debug('Dropping table ' . $table, PEAR_LOG_DEBUG);
        $this->dal->execute($query, $result);
    }
    
    /**
     * A method for dropping temporary tables.
     *
     * @param string $table The temporary table to drop.
     */
    function dropTempTable($table)
    {
        $query = 'DROP TEMPORARY TABLE ' . $table;
        debug('Dropping temporary table ' . $table, PEAR_LOG_DEBUG);
        $this->dal->execute($query, $result);
    }
    
}

?>